@@ -25,7 +25,7 @@ recode_num <- function(X, ..., default = NULL, missing = NULL, set = FALSE) {
         repfun <- function(y) if(is.numeric(y)) scv(y, nam, args, set) else y # `[<-`(y, y == nam, value = args)
       }
     } else {
-      nr <- if(is.atomic(X)) NROW(X) else fnrow2(X)
+      nr <- if(is.atomic(X)) NROW(X) else fnrow(X)
       if(missingl) {
         repfun <- function(y) if(is.numeric(y)) {
           z <- scv(y, NA, default, set, TRUE) # duplAttributes(alloc(default, nr), y)
@@ -54,7 +54,7 @@ recode_num <- function(X, ..., default = NULL, missing = NULL, set = FALSE) {
         } else y
       }
     } else {
-      nr <- if(is.atomic(X)) NROW(X) else fnrow2(X)
+      nr <- if(is.atomic(X)) NROW(X) else fnrow(X)
       if(missingl) {
         repfun <- function(y) if(is.numeric(y)) {
           z <- scv(y, NA, default, set, TRUE) # duplAttributes(alloc(default, nr), y)
@@ -102,7 +102,7 @@ recode_char <- function(X, ..., default = NULL, missing = NULL, regex = FALSE,
           repfun <- function(y) if(is.character(y)) `[<-`(y, grepl(nam, y, ignore.case, FALSE, fixed), value = args) else y
         }
       } else {
-        nr <- if(is.atomic(X)) NROW(X) else fnrow2(X)
+        nr <- if(is.atomic(X)) NROW(X) else fnrow(X)
         if(missingl) {
           repfun <- function(y) if(is.character(y)) {
             z <- scv(y, NA, default, set, TRUE)  # duplAttributes(alloc(default, nr), y)
@@ -131,7 +131,7 @@ recode_char <- function(X, ..., default = NULL, missing = NULL, regex = FALSE,
           } else y
         }
       } else {
-        nr <- if(is.atomic(X)) NROW(X) else fnrow2(X)
+        nr <- if(is.atomic(X)) NROW(X) else fnrow(X)
         if(missingl) {
           repfun <- function(y) if(is.character(y)) {
             z <- scv(y, NA, default, set, TRUE)  # duplAttributes(alloc(default, nr), y)
@@ -161,7 +161,7 @@ recode_char <- function(X, ..., default = NULL, missing = NULL, regex = FALSE,
           repfun <- function(y) if(is.character(y)) scv(y, nam, args, set) else y # `[<-`(y, y == nam, value = args)
         }
       } else {
-        nr <- if(is.atomic(X)) NROW(X) else fnrow2(X)
+        nr <- if(is.atomic(X)) NROW(X) else fnrow(X)
         if(missingl) {
           repfun <- function(y) if(is.character(y)) {
             z <- scv(y, NA, default, set, TRUE)  # duplAttributes(alloc(default, nr), y)
@@ -190,7 +190,7 @@ recode_char <- function(X, ..., default = NULL, missing = NULL, regex = FALSE,
           } else y
         }
       } else {
-        nr <- if(is.atomic(X)) NROW(X) else fnrow2(X)
+        nr <- if(is.atomic(X)) NROW(X) else fnrow(X)
         if(missingl) {
           repfun <- function(y) if(is.character(y)) {
             z <- scv(y, NA, default, set, TRUE)  # duplAttributes(alloc(default, nr), y)
@@ -346,7 +346,7 @@ pad_atomic <- function(x, i, n, value) {
 pad <- function(X, i, value = NA, method = c("auto", "xpos", "vpos")) { # 1 - i is same length as X, fill missing, 2 - i is positive: insert missing values in positions
   ilog <- is.logical(i)
   ineg <- i[1L] < 0L
-  n <- if(is.list(X)) length(.subset2(X, 1L)) else if(is.matrix(X)) dim(X)[1L] else length(X)
+  n <- if(is.list(X) || is.matrix(X)) fnrow(X) else length(X)
   xpos <- switch(method[1L], auto = if(ilog) bsum(i) == n else if(ineg) FALSE else length(i) == n,
                  xpos = TRUE, vpos = FALSE, stop("Unknown method: ", method[1L]))
   n <- if(ilog) length(i) else if(xpos && !ineg) bmax(i) else n + length(i)
@@ -365,232 +365,3 @@ pad <- function(X, i, value = NA, method = c("auto", "xpos", "vpos")) { # 1 - i
 }
 
 # Something like this already exists?? -> should work with lists as well...
-
-
-# # Previous version of Recode (Until collapse 1.1.0), Now depreciated in favor or recode_num and recode_char
-# comp <- function(x, val) do.call(cbind, lapply(x, `==`, val))
-# comp_grepl <- function(x, val) do.call(cbind, lapply(x, function(y) grepl(val, y)))
-#
-# Recode <- function(X, ..., copy = FALSE, reserve.na.nan = TRUE, regex = FALSE) {
-#   .Deprecated("recode_num, recode_char")
-#   if(missing(...)) stop("Recode requires arguments of the form: value = replacement")
-#   if(is.list(X) && !inherits(X, "data.frame")) stop("Recode only works with atomic objects or data.frames")
-#   args <- list(...)
-#   nam <- names(args)
-#
-#   if(reserve.na.nan && any(wm <- !is.na(ma <- match(c("NaN","NA"), nam)))) { # more efficient way to code this ??
-#     if(wm[1L]) {  # note: does not give multiple-matching error !!
-#       if(is.list(X)) {
-#         X[do.call(cbind, lapply(X, is.nan))] <- args[[ma[1L]]]
-#       } else X[is.nan(X)] <- args[[ma[1L]]]
-#     }
-#     if(wm[2L]) X[is.na(X)] <- args[[ma[2L]]] # is.na already accounts for NaN, so this needs to the the order!!
-#     if(bsum(wm) == length(args)) return(X)
-#     args <- args[-ma[wm]]
-#     nam <- names(args)
-#   }
-#
-#   arglen <- length(args)
-#   onearg <- arglen == 1L
-#
-#   if(regex) {
-#     if(is.list(X)) {
-#       if(onearg) {
-#         X[comp_grepl(X, nam)] <- args[[1L]]
-#       } else if(copy) {
-#         for (i in seq_along(X)) {
-#           Xi <- X[[i]]
-#           for (j in seq_len(arglen)) X[[i]][grepl(nam[j], Xi)] <- args[[j]]
-#         }
-#       } else for (j in seq_len(arglen)) X[comp_grepl(X, nam[j])] <- args[[j]]
-#     } else {
-#       if(onearg) X[grepl(nam, X)] <- args[[1L]] else if(copy) {
-#         Y <- X
-#         for (j in seq_len(arglen)) X[grepl(nam[j], Y)] <- args[[j]]
-#       } else for (j in seq_len(arglen)) X[grepl(nam[j], X)] <- args[[j]]
-#     }
-#   } else {
-#     if(is.list(X)) {
-#       oldopts <- options(warn = -1) # faster than suppressWarnings !!
-#       on.exit(options(oldopts))
-#       numnam <- as.numeric(nam) # suppressWarnings(numnam <- as.numeric(nam)) -> this line takes most time !!
-#       if(onearg && !is.na(numnam)) nam <- numnam else {
-#         nam <- as.vector(nam, "list")
-#         if(any(numnaml <- !is.na(numnam))) nam[numnaml] <- numnam[numnaml]
-#       }
-#       if(onearg) X[comp(X, nam)] <- args[[1L]] else if(copy) {
-#         Y <- X
-#         for (j in seq_len(arglen)) X[comp(Y, nam[[j]])] <- args[[j]]
-#       } else for (j in seq_len(arglen)) X[comp(X, nam[[j]])] <- args[[j]]
-#     } else {
-#       if(!is.character(X)) nam <- as.numeric(nam)
-#       if(onearg) X[X == nam] <- args[[1L]] else if(copy) {
-#         Y <- X
-#         for (j in seq_len(arglen)) X[Y == nam[j]] <- args[[j]]
-#       } else for (j in seq_len(arglen)) X[X == nam[j]] <- args[[j]]
-#     }
-#   }
-#   return(X)
-# }
-
-
-# Experimental:
-# recode_num2 is slightly slower than recode_num above .... (because 2 times apply) but almost identical..
-# recode_num2 <- function(X, ...) { # , regex = FALSE # , default = NULL
-#   if(missing(...)) stop("Recode requires arguments of the form: value = replacement")
-#   args <- list(...)
-#   nam <- as.numeric(names(args))
-#   if(anyNA(nam)) stop(paste("Non-numeric arguments:", paste(names(args)[is.na(nam)], collapse = ", ")))
-#   arglen <- length(args)
-#
-#   if(arglen == 1L) {
-#     args <- args[[1L]]
-#     repfun <- function(y) `[<-`(y, y == nam, value = args)
-#   } else {
-#     seqarg <- seq_len(arglen)
-#     repfun <- function(y) {
-#       z <- y
-#       for(i in seqarg) z[y == nam[i]] <- args[[i]]
-#       z
-#     }
-#   }
-#   if(is.list(X)) {
-#     num <- vapply(unattrib(X), is.numeric, TRUE)
-#     clx <- class(X)
-#     class(X) <- NULL
-#     X[num] <- lapply(X[num], repfun)
-#     return(`oldClass<-`(X, clx))
-#   }
-#   if(!is.numeric(X)) stop("X needs to be numeric or a list")
-#   return(repfun(X))
-# }
-#
-#
-#
-# # possibly even faster by converting df to matrix and back for multiple comp ??
-# Recode2 <- function(X, ...) { # , regex = FALSE # , default = NULL
-#   if(missing(...)) stop("Recode requires arguments of the form: value = replacement")
-#   args <- list(...)
-#   nam <- names(args)
-#   arglen <- length(args)
-#   onearg <- arglen == 1L
-#
-#   if(onearg) {
-#     args <- args[[1L]]
-#     repfun <- function(y) {
-#       if(is.numeric(y)) {
-#         v <- as.numeric(nam)
-#         if(is.na(v)) {
-#           warning("Trying to replace a non-numeric expressiom in a numeric column: Skipping this column")
-#           return(y)
-#         }
-#         `[<-`(y, y == v, value = args)
-#       } else `[<-`(y, y == nam, value = args)
-#     }
-#   } else {
-#     seqarg <- seq_len(arglen)
-#     repfun <- function(y) {
-#       z <- y
-#       if(is.numeric(y)) {
-#         if(is.null(v)) {
-#           v <<- as.numeric(nam)
-#           if(anyNA(v)) {
-#             warning("Trying to replace a non-numeric expressiom in a numeric column: Skipping those expressions")
-#             v <- na_rm(v)
-#             if(!length(v)) return(y)
-#           }
-#         }
-#         for(i in seqarg) z[y == v[i]] <- args[[i]]
-#       } else {
-#         for(i in seqarg) z[y == nam[i]] <- args[[i]]
-#       }
-#       z
-#     }
-#   }
-#
-#   if(is.list(X)) {
-#     return(duplAttributes(lapply(unattrib(X), repfun), X))
-#   }
-#   if(!is.character(X)) storage.mode(nam) <- storage.mode(X)
-#   return(repfun(X))
-# }
-#
-#
-# rec1 <- function(x, ...) {
-#   args <- list(...)
-#   nam <- as.numeric(names(args))
-#   ax <- attributes(x)
-#   attributes(x) <- NULL
-#   for(i in seq_along(args)) {
-#     ni <- nam[i] # Faster!
-#     argi <- args[[i]]
-#     x <- lapply(x, function(y) `[<-`(y, y == ni, value = argi))
-#   }
-#   setAttributes(x, ax)
-# }
-# # More memory efficient !!
-# rec2 <- function(x, ...) {
-#   args <- list(...)
-#   nam <- as.numeric(names(args))
-#   if(length(args) > 1L) repfun <- function(y) {
-#     for(i in seq_along(args)) y[y == nam[i]] <- args[[i]]
-#     y
-#   } else repfun <- function(y) `[<-`(y, y == nam, value = args[[1L]])
-#   dapply(x, repfun)
-# }
-# # Even more memory efficient and faster !!
-# rec3 <- function(x, ...) {
-#   args <- list(...)
-#   nam <- as.numeric(names(args))
-#   if(length(args) > 1L) repfun <- function(y) {
-#     z <- y
-#     for(i in seq_along(args)) z[y == nam[i]] <- args[[i]]
-#     z
-#   } else repfun <- function(y) `[<-`(y, y == nam, value = args[[1L]])
-#   dapply(x, repfun)
-# }
-#
-#
-# tf <- function(x, ...) list(...)
-#
-# # profvis({
-# #   v <- c("a","b","c")
-# #   Recode(v, a = "b")
-# # })
-
-
-
-
-# Previous Versions (until collapse 1.1.0)
-
-# replace_outliers <- function(X, limits, value = NA, single.limit = c("SDs","Min","Max")) {
-#   if(length(limits) > 1L) {
-#     betw <- function(x, lim) x < lim[1L] | x > lim[2L]
-#     if(is.list(X))  {
-#       for (i in seq_along(X)) X[[i]][betw(X[[i]], limits)] <- value
-#     } else X[betw(X, limits)] <- value
-#     return(X)
-#   } else {
-#     switch(single.limit[1L],
-#            SDs = {
-#              if(is.list(X))  {
-#                gsd <- function(x, lim) abs(x) > lim*fsdCpp(x)
-#                for (i in seq_along(X)) X[[i]][gsd(X[[i]], limits)] <- value
-#              } else X[abs(X) > lim*fsd(X)] <- value
-#              return(X)
-#            },
-#            Min = {
-#              if(is.list(X))  {
-#                for (i in seq_along(X)) X[[i]][X[[i]] < limits] <- value
-#              } else X[X < limits] <- value
-#              return(X)
-#            },
-#            Max = {
-#              if(is.list(X))  {
-#                for (i in seq_along(X)) X[[i]][X[[i]] > limits] <- value
-#              } else X[X > limits] <- value
-#              return(X)
-#            },
-#            stop("Unknown single.limit option"))
-#   }
-# }
