@@ -4,8 +4,7 @@ bprod <- base::prod
 bmin <- base::min
 bmax <- base::max
 
-# Row-operations (documented under data transformations...) ... see if any other package has it (i.e. matrixStats etc..)
-# or wirhout r ??? look for %+% function on Rducumentation.. rdio.
+# Row-operations (documented under data transformations...) ...
 
 "%rr%" <- function(X, v) if(is.atomic(X) || is.atomic(v) || inherits(X, "data.frame")) TRA(X, v, "replace_fill") else # outer(rep.int(1L, dim(X)[2L]), v)
   duplAttributes(.mapply(function(x, y) TRA(x, y, "replace_fill"), list(unattrib(X), unattrib(v)), NULL), X)
@@ -19,8 +18,6 @@ bmax <- base::max
   duplAttributes(.mapply(function(x, y) TRA(x, y, "/"), list(unattrib(X), unattrib(v)), NULL), X)
 
 
-
-# othidentity <- function(x, y) y
 "%cr%" <- function(X, V) if(is.atomic(X)) return(duplAttributes(rep(V, NCOL(X)), X)) else # outer(rep.int(1L, dim(X)[2L]), V)
   if(is.atomic(V)) return(duplAttributes(lapply(vector("list", length(unclass(X))), function(z) V), X)) else
     copyAttrib(V, X) # copyAttrib first makes a shallow copy of V
@@ -233,7 +230,7 @@ rm_stub <- function(X, stub, pre = TRUE, regex = FALSE, cols = NULL, ...) {
 
 setRownames <- function(object, nm = if(is.atomic(object)) seq_row(object) else NULL) {
   if(is.list(object)) {
-    l <- length(.subset2(object, 1L))
+    l <- .Call(C_fnrow, object)
     if(is.null(nm)) nm <- .set_row_names(l) else if(length(nm) != l) stop("supplied row-names must match list extent")
     attr(object, "row.names") <- nm
     if(inherits(object, "data.table")) return(alc(object))
@@ -317,13 +314,13 @@ addAttributes <- function(x, a) .Call(C_setAttributes, x, c(attributes(x), a))
 
 is_categorical <- function(x) !is.numeric(x)
 is.categorical <- function(x) {
-  message("Note that 'is.categorical' was renamed to 'is_categorical'. It will not be removed anytime soon, but please use updated function names in new code, see help('collapse-renamed')")
+  .Deprecated(msg = "'is.categorical' was renamed to 'is_categorical'. It will be removed end of 2023, see help('collapse-renamed').")
   !is.numeric(x)
 }
 
 is_date <- function(x) inherits(x, c("Date","POSIXlt","POSIXct"))
 is.Date <- function(x) {
-  message("Note that 'is.Date' was renamed to 'is_date'. It will not be removed anytime soon, but please use updated function names in new code, see help('collapse-renamed')")
+  .Deprecated(msg = "'is.Date' was renamed to 'is_date'. It will be removed end of 2023, see help('collapse-renamed').")
   inherits(x, c("Date","POSIXlt","POSIXct"))
 }
 
@@ -348,6 +345,7 @@ whichNA <- function(x, invert = FALSE) .Call(C_whichv, x, NA, invert)
 frange <- function(x, na.rm = TRUE) .Call(C_frange, x, na.rm)
 alloc <- function(value, n) .Call(C_alloc, value, n)
 vgcd <- function(x) .Call(C_vecgcd, x)
+fdist <- function(x, v = NULL, ..., method = "euclidean", nthreads = 1L) .Call(C_fdist, if(is.atomic(x)) x else qM(x), v, method, nthreads)
 
 allNA <- function(x) .Call(C_allNA, x, TRUE) # True means give error for unsupported vector types, not FALSE.
 anyv <- function(x, value) .Call(C_anyallv, x, value, FALSE)
@@ -387,11 +385,25 @@ setop <- function(X, op, V, ..., rowwise = FALSE) # Making sure some error is pr
 "%*=%" <- function(X, V) invisible(.Call(C_setop, X, V, 3L, FALSE))
 "%/=%" <- function(X, V) invisible(.Call(C_setop, X, V, 4L, FALSE))
 
+# Internal functions
+missDF <- function(x, cols = seq_along(unclass(x))) .Call(C_dt_na, x, cols, 0, FALSE)
+frowSums <- function(x) {
+  nr <- dim(x)[1L]
+  .rowSums(x, nr, length(x)/nr)
+}
+fcolSums <- function(x) {
+  nr <- dim(x)[1L]
+  .colSums(x, nr, length(x)/nr)
+}
 
-missing_cases <- function(X, cols = NULL) {
-  if(is.list(X)) return(.Call(C_dt_na, X, if(is.null(cols)) seq_along(unclass(X)) else cols2int(cols, X, attr(X, "names"))))
-  if(is.matrix(X)) return(if(is.null(cols)) !complete.cases(X) else !complete.cases(X[, cols]))
-  is.na(X)
+missing_cases <- function(X, cols = NULL, prop = 0, count = FALSE) {
+  if(is.list(X)) return(.Call(C_dt_na, X, if(is.null(cols)) seq_along(unclass(X)) else cols2int(cols, X, attr(X, "names")), prop, count))
+  if(is.matrix(X)) {
+    if(length(cols)) X <- X[, cols]
+    if(is.matrix(X)) return(if(count) as.integer(frowSums(is.na(X))) else if(prop > 0) # as.integer() needed to establish consistency (integer output)
+      frowSums(is.na(X)) >= bmax(as.integer(prop * NCOL(X)), 1L) else !complete.cases(X))
+  }
+  if(count) as.integer(is.na(X)) else is.na(X) # Note: as.integer() here is inefficient, but storage.mode() <- "integer" is also. Would have to export a R wrapper to C function SET_TYPEOF()... but this is probably never invoked anyway.
 }
 
 na_rm <- function(x) .Call(C_na_rm, x)  # x[!is.na(x)]
@@ -400,13 +412,12 @@ null_rm <- function(l) if(!all(ind <- vlengths(l, FALSE) > 0L)) .subset(l, ind)
 
 all_eq <- function(x) .Call(C_anyallv, x, x[1L], TRUE)
 
-na_omit <- function(X, cols = NULL, na.attr = FALSE, ...) {
+na_omit <- function(X, cols = NULL, na.attr = FALSE, prop = 0, ...) {
   if(is.list(X)) {
     iX <- seq_along(unclass(X))
-    rl <- if(is.null(cols)) .Call(C_dt_na, X, iX) else
-          .Call(C_dt_na, X, cols2int(cols, X, attr(X, "names"))) # gives error if X not list
+    rl <- .Call(C_dt_na, X, if(is.null(cols)) iX else cols2int(cols, X, attr(X, "names")), prop, FALSE)
     rkeep <- whichv(rl, FALSE)
-    if(length(rkeep) == fnrow2(X)) return(condalc(X, inherits(X, "data.table")))
+    if(length(rkeep) == fnrow(X)) return(condalc(X, inherits(X, "data.table")))
     res <- .Call(C_subsetDT, X, rkeep, iX, FALSE) # This allocates data.tables...
     rn <- attr(X, "row.names")
     if(!(is.numeric(rn) || is.null(rn) || rn[1L] == "1")) attr(res, "row.names") <- Csv(rn, rkeep)
@@ -421,7 +432,8 @@ na_omit <- function(X, cols = NULL, na.attr = FALSE, ...) {
       attr(res, "index") <- index_omit
     }
   } else {
-    rl <- if(is.null(cols)) complete.cases(X) else complete.cases(X[, cols])
+    Xcols <- if(is.null(cols)) X else X[, cols]
+    rl <- if(prop > 0 && is.matrix(Xcols)) frowSums(is.na(Xcols)) < bmax(as.integer(prop * ncol(Xcols)), 1L) else complete.cases(Xcols)
     rkeep <- which(rl)
     if(length(rkeep) == NROW(X)) return(X)
     res <- if(is.matrix(X)) X[rkeep, , drop = FALSE, ...] else X[rkeep, ...]
@@ -432,7 +444,7 @@ na_omit <- function(X, cols = NULL, na.attr = FALSE, ...) {
 
 na_insert <- function(X, prop = 0.1, value = NA) {
   if(is.list(X)) {
-    n <- fnrow2(X)
+    n <- fnrow(X)
     nmiss <- floor(n * prop)
     res <- duplAttributes(lapply(unattrib(X), function(y) `[<-`(y, sample.int(n, nmiss), value = value)), X)
     return(if(inherits(X, "data.table")) alc(res) else res)
@@ -449,9 +461,7 @@ fnlevels <- function(x) length(attr(x, "levels"))
 
 # flevels <- function(x) attr(x, "levels")
 
-fnrow <- function(X) if(is.list(X)) length(.subset2(X, 1L)) else dim(X)[1L]
-
-fnrow2 <- function(X) length(.subset2(X, 1L))
+fnrow <- function(X) .Call(C_fnrow, X)  # if(is.list(X)) length(.subset2(X, 1L)) else dim(X)[1L]
 
 fncol <- function(X) if(is.list(X)) length(unclass(X)) else dim(X)[2L]
 
@@ -459,11 +469,10 @@ fNCOL <- function(X) if(is.list(X)) length(unclass(X)) else NCOL(X)
 
 fdim <- function(X) {
    if(is.atomic(X)) return(dim(X)) # or if !is.list ?
-   oldClass(X) <- NULL
-   c(length(X[[1L]]), length(X)) # Faster than c(length(.subset2(X, 1L)), length(unclass(X)))
+   c(.Call(C_fnrow, X), length(unclass(X)))
 }
 
-seq_row <- function(X) if(is.list(X)) seq_along(.subset2(X, 1L)) else seq_len(dim(X)[1L])
+seq_row <- function(X) seq_len(.Call(C_fnrow, X))
 
 seq_col <- function(X) if(is.list(X)) seq_along(unclass(X)) else seq_len(dim(X)[2L])
 
@@ -500,12 +509,12 @@ as_character_factor <- function(X, keep.attr = TRUE) {
 }
 
 as.numeric_factor <- function(X, keep.attr = TRUE) {
-  message("Note that 'as.numeric_factor' was renamed to 'as_numeric_factor'. It will not be removed anytime soon, but please use updated function names in new code, see help('collapse-renamed')")
+  .Deprecated(msg = "'as.numeric_factor' was renamed to 'as_numeric_factor'. It will be removed end of 2023, see help('collapse-renamed').")
   as_numeric_factor(X, keep.attr)
 }
 
 as.character_factor <- function(X, keep.attr = TRUE) {
-  message("Note that 'as.character_factor' was renamed to 'as_character_factor'. It will not be removed anytime soon, but please use updated function names in new code, see help('collapse-renamed')")
+  .Deprecated(msg = "'as.character_factor' was renamed to 'as_character_factor'. It will be removed end of 2023, see help('collapse-renamed').")
   as_character_factor(X, keep.attr)
 }
 
@@ -673,11 +682,6 @@ charorNULL <- function(x) if(is.character(x)) x else NULL
 
 tochar <- function(x) if(is.character(x)) x else as.character(x)  # if(is.object(x)) as.character(x) else .Call(C_aschar, x)
 
-# more security here?
-# unique_factor <- function(x) {  # Still needed with new collap solution ? -> Nope !
-#   res <- seq_along(attr(x, "levels"))
-#   .Call(C_duplAttributes, res, x)
-# }
 # dotstostr <- function(...) {
 #   args <- deparse(substitute(c(...)))
 #   nc <- nchar(args)
@@ -705,9 +709,10 @@ addNA2 <- function(x) {
   oldClass(x) <- NULL
   if(!anyNA(lev <- attr(x, "levels"))) {
     attr(x, "levels") <- c(lev, NA_character_)
-    x[is.na(x)] <- length(lev) + 1L
-  } else x[is.na(x)] <- length(lev)
-  `oldClass<-`(x, clx)
+    .Call(C_setcopyv, x, NA_integer_, length(lev) + 1L, FALSE, TRUE, FALSE) # x[is.na(x)] <- length(lev) + 1L
+  } else .Call(C_setcopyv, x, NA_integer_, length(lev), FALSE, TRUE, FALSE) # x[is.na(x)] <- length(lev)
+  oldClass(x) <- clx
+  x
 }
 
 # addNA2 <- function(x) {
@@ -718,7 +723,6 @@ addNA2 <- function(x) {
 #   return(`oldClass<-`(factor(x, levels = ll, exclude = NULL), clx))
 # }
 
-
 l1orn <- function(x, nam) if(length(x) == 1L) x else nam
 l1orlst <- function(x) if(length(x) == 1L) x else x[length(x)]
 
@@ -729,25 +733,8 @@ fsimplify2array <- function(l) {
   res
 }
 
-
 # fss <- function(x, i, j) {
 #   rn <- attr(x, "row.names")
 #   if(is.numeric(rn) || is.null(rn) || rn[1L] == "1") return(.Call(C_subsetDT, x, i, j))
 #   return(`attr<-`(.Call(C_subsetDT, x, i, j), "row.names", rn[r]))
 # }
-
-# fsplit_DF <- function(x, j, f, rnl, ...) {
-#   j <- seq_along(unclass(x))
-#   rn <- attr(x, "row.names")
-#   if(is.numeric(rn) || is.null(rn) || rn[1L] == "1")
-#     return(lapply(split.default(seq_along(.subset2(x, 1L)), f, ...),
-#                   function(i) .Call(C_subsetDT, x, i, j)))
-#   lapply(split.default(seq_along(.subset2(x, 1L)), f, ...),
-#          function(i) `attr<-`(.Call(C_subsetDT, x, i, j), "row.names", rn[i]))
-# }
-
-
-
-
-
-
